﻿/*
 * Weihnachtsstern.s
 * Scott-Falk Hühn
 * Version 1.11, 12.12.2021
 *
 * Byte-Reihenfolge: LED1G-LED1R-LED1B-LED2G-LED2R-LED2B...
 * Bit-Reihenfolge: G7-G6-G5-G4-G3-G2-G1-G0-R7-R6-R5-R4-R3-R2-R1-R0-B7-B6-B5-B4-B3-B2-B1-B0...
 *
 * Ausgabe mit 800kHz (1,25µs):
 * 0-Bit:  5 Taktzyklen High, 15 Taktzyklen Low (312ns/938ns, Länge 1,25µs)
 * 1-Bit: 13 Taktzyklen High,  7 Taktzyklen Low (812ns/438ns, Länge 1,25µs)
 * Gesamt-Dauer einer kompletten Ausgabe: 1,5ms (1,25µs * 8 * 3 * 50)
 *
 * Parameter 1 (r24, r25): Zeiger auf LED-Farbwert-Tabelle
 * Parameter 2 (r22, r23): Größe der LED-Farbwert-Tabelle (Anzahl der auszugebenden Bytes)
 */

#include <avr/io.h>                                     // I/O-Definitionen laden
#define led_port PORTB                                  // LED-Ausgabeport festlegen
#define led_pin PINB2                                   // LED-Ausgabe-Anschluss festlegen

.global	led_out                                         // Definition als globale Funktion

led_out:
        movw    r30, r24                                // Zeiger auf LED-Farbwert-Tabelle kopieren
        movw	r26, r22				// Anzahl der Bytes kopieren
        cli                                             // Interrupts ausschalten
;
led010: ldi     r25, 8                                  // 8 Bits ausgeben
        ld      r24, z+                                 // Byte aus LED-Farbwert-Tabelle holen
;
led020: sbi     _SFR_IO_ADDR(led_port), led_pin         // Ausgang auf High setzen
        rol     r24                                     // Farbbyte verschieben
        nop                                             // warten
        brcs    led030                                  // 1-Bit ausgeben? ja -> weiter
        cbi     _SFR_IO_ADDR(led_port), led_pin         // sonst 0-Bit, Ausgang wieder auf Low setzen, 5 Taktzyklen High
        nop                                             // es folgen 15 Taktzyklen Low
        nop                                             // warten
        nop
        nop
        dec     r25                                     // alle Bits ausgegeben?
        brne    led040                                  // nein -> Schleife
        sbiw    r26, 1                                  // alle Bytes ausgegeben?
        brne    led010                                  // nein -> Schleife
        sei                                             // sonst Interrupts wieder einschalten
        ret                                             // Ende
;
led030: nop                                             // 1-Bit ausgeben, warten
        nop
        dec     r25                                     // alle Bits ausgegeben?
        breq    led060                                  // ja -> weiter
        nop                                             // warten
        nop
        nop
        cbi     _SFR_IO_ADDR(led_port), led_pin         // Ausgang wieder auf Low setzen, 13 Taktzyklen High
        rjmp	led050                                  // es folgen 7 Taktzyklen Low
led040: nop                                             // warten
        nop
        nop
led050:	nop
        rjmp    led020                                  // Schleife
;
led060: sbiw    r26, 1                                  // alle Bytes ausgegeben? Abfrage erfolgt später
        cbi     _SFR_IO_ADDR(led_port), led_pin         // Ausgang wieder auf Low setzen, 13 Taktzyklen High
        brne    led010                                  // alle Bytes ausgegeben? nein -> Schleife
        sei                                             // sonst Interrupts wieder einschalten
        ret                                             // Ende
